﻿using UnityEngine.TestTools;
using System.Collections;
using TrackAnalysis;
using UnityEngine;
using JudgeModule;
using NUnit.Framework;
using UnityEngine.UI;

public class NoteJudgerTests
{
    [UnityTest]
    public IEnumerator DeactivedNote_Should_Change_Parent_Deactives()
    {
        var note = CreateNoteAndController("SBT", 0);

        var judger = new NoteJudger(x => { });

        yield return null;

        judger.DeactivateNote(note);

        var expected = "Deactives";
        var actual = note.Component.transform.parent.name;

        Assert.AreEqual(expected, actual, "Deactivednote should change parent of note to deactives");
    }

    [UnityTest]
    public IEnumerator SetJudge_Should_Change_Judge_Text()
    {
        var text = CreateJudgeText();

        CreateNoteAndController("SBT", 0);

        var judger = new NoteJudger(x => { });

        int cnt = 0;
        foreach (var x in Judge.JudgeList)
        {
            yield return null;

            judger.SetJudge(x);

            if (text.text == x.Name &&
                text.color == x.Color)
                ++cnt;
        }

        var expected = Judge.JudgeList.Count;
        var actual = cnt;

        Assert.AreEqual(expected, actual, "SetJudge should change judge text's text and color");
    }

    [UnityTest]
    public IEnumerator WrongNoteProc_Should_Execute_EndGame_When_Button_Note()
    {
        var note = CreateNoteAndController("SBT", 0);

        CreateJudgeText();

        bool isEnd = false;
        var judger = new NoteJudger(x => isEnd = true);

        yield return null;

        judger.WrongNoteProc(note);

        var expected = true;
        var actual = isEnd;

        Assert.AreEqual(expected, actual, "WrongNoteProc should execute endgmae when button note");
    }

    [UnityTest]
    public IEnumerator WrongNoteProc_Should_Not_Execute_EndGame_When_Motion_Note()
    {
        var note = CreateNoteAndController("SMO", 0);

        CreateJudgeText();

        bool isEnd = false;
        var judger = new NoteJudger(x => isEnd = true);

        yield return null;

        judger.WrongNoteProc(note);

        var expected = false;
        var actual = isEnd;

        Assert.AreEqual(expected, actual, "WrongNoteProc should not execute endgmae when motion note");
    }

    [UnityTest]
    public IEnumerator EnteredNoteProc_Should_Change_Activated_True_When_Long_Note()
    {
        var note = CreateNoteAndController("LBT", 0, Judge.MISS.ButtonTimingRange + 10);

        CreateJudgeText();
        
        var judger = new NoteJudger(x => { });

        yield return null;

        judger.EnteredNoteProc(note, note.StartTiming);

        var expected = true;
        var actual = note.Activated;

        Assert.AreEqual(expected, actual, "WrongNoteProc should change activated to true when long note");
    }

    [UnityTest]
    public IEnumerator EnteredNoteProc_Should_Not_Change_Activated_True_When_Short_Note()
    {
        var note = CreateNoteAndController("SBT", 0);

        CreateJudgeText();

        var judger = new NoteJudger(x => { });

        yield return null;

        judger.EnteredNoteProc(note, note.StartTiming);

        var expected = false;
        var actual = note.Activated;

        Assert.AreEqual(expected, actual, "WrongNoteProc should not change activated to true when short note");
    }

    [UnityTest]
    public IEnumerator ContinuingNoteProc_Should_Deactive_When_Not_End_And_Break()
    {
        var note = CreateNoteAndController("LBT", 0, Judge.BAD.ButtonTimingRange + 1);

        CreateJudgeText();

        var judger = new NoteJudger(x => { });

        yield return null;

        judger.ContinuingNoteProc(note, -(Judge.BAD.ButtonTimingRange + 1));

        var expected = "Deactives";
        var actual = note.Component.transform.parent.name;

        Assert.AreEqual(expected, actual, "ContinuingNoteProc should deactive when not end and break");
    }

    [UnityTest]
    public IEnumerator ContinuingNoteProc_Should_Not_Deactive_When_End()
    {
        var note = CreateNoteAndController("LBT", 0, 1);

        CreateJudgeText();

        var judger = new NoteJudger(x => { });

        yield return null;

        judger.ContinuingNoteProc(note, note.EndTiming);

        var expected = "NoteObj";
        var actual = note.Component.transform.parent.name;

        Assert.AreEqual(expected, actual, "ContinuingNoteProc should not deactive when end");
    }

    [UnityTest]
    public IEnumerator ContinuingNoteProc_Should_Not_Deactive_When_Not_Break()
    {
        var note = CreateNoteAndController("LBT", 0, Judge.BAD.ButtonTimingRange + 1);

        CreateJudgeText();

        var judger = new NoteJudger(x => { });

        yield return null;

        judger.ContinuingNoteProc(note, note.StartTiming);

        var expected = "NoteObj";
        var actual = note.Component.transform.parent.name;

        Assert.AreEqual(expected, actual, "ContinuingNoteProc should not deactive when not break");
    }

    private Note CreateNoteAndController(string type, float start, float end = 0)
    {
        var noteobj = new GameObject("NoteObj");
        var deactives = new GameObject("Deactives");

        var notemaker = new NoteMaker();

        Note note;
        switch (type)
        {
            case "LBT":
                notemaker.Make("AA", "LBT", start);
                note = notemaker.Make("AA", "LBT", end);
                break;
            case "SMO":
                note = notemaker.Make("CP", "SMO", start);
                break;
            case "LMO":
                notemaker.Make("JS", "LMO", start);
                note = notemaker.Make("JS", "LMO", end);
                break;
            default:
                note = notemaker.Make("AA", "SBT", start);
                break;
        }
        
        var obj = new GameObject();
        obj.transform.SetParent(noteobj.transform);

        var controller = obj.AddComponent<Controller>();

        controller.Instance = note;
        note.Component = controller;

        return note;
    }

    private Text CreateJudgeText()
    {
        var judgeText = new GameObject("Judge", typeof(RectTransform));

        return judgeText.AddComponent<Text>();
    }
}